// 
// PSP_Bot by Geecko
//
// This work is licensed under the Creative Commons BY-NC-SA 3.0 Unported License.
// See LICENSE for more details.

#include <stdio.h>

#include "parsing.h"
#include "language.h"
#include "common.h"
#include "level.h"
#include "game.h"

Level lvl;


void loadLevel()
{
  int i, j, tmp;
  char buf[1024], buf2[1024];
  sprintf(buf,"levels/%d.hlf",lvl.nbr);
  loadFile(buf);

  // Get the level introduction text
  readFromFile("en",buf);
  sscanf(buf,"\"%[^\"]\"",text.game.intro);

  // Set function highlighting
  readFromFile("highlight",buf);
  for (i=0; i!=FUNC_NBR; i++) lvl.highlight[i] = 0; // Reset
  for (i=0; i!=strlen(buf); i++) lvl.highlight[buf[i]-'0'] = 1;

  // Robot properties
  readFromFile("start_x",buf);
  sscanf(buf,"%hd",&lvl.x);
  readFromFile("start_y",buf);
  sscanf(buf,"%hd",&lvl.y);
  readFromFile("start_dir",buf);
  sscanf(buf,"%hd",&lvl.dir);
  initRobot();

  // Heightmap properties
  lvl.cumSumHeight = 0;
  for (j=0; j!=LEVEL_SIDE; j++)
  {
    sprintf(buf2,"h%d",j);
    readFromFile(buf2,buf);
    for (i=0; i!=LEVEL_SIDE; i++)
    {
      sscanf(buf+i,"%1d",&tmp);
      if (tmp >= 0) lvl.map[i][j].h = tmp, lvl.cumSumHeight += tmp;
      else lvl.map[i][j].h = 0;
    }
  }

  // Square types
  for (j=0; j!=LEVEL_SIDE; j++)
  {
    sprintf(buf2,"t%d",j);
    readFromFile(buf2,buf);
    for (i=0; i!=LEVEL_SIDE; i++)
    {
      sscanf(buf+i,"%1d",&tmp);
      if (tmp >= 0 && tmp <= 1) lvl.map[i][j].state = tmp;
      else lvl.map[i][j].state = 0;
    }
  }

  closeFile();
}


void resetLevel()
{
  int i, j;
  for (j=0; j!=LEVEL_SIDE; j++)
    for (i=0; i!=LEVEL_SIDE; i++)
      if (lvl.map[i][j].state == 2) lvl.map[i][j].state = 1;
}


short nextLevel()
{
  if (++lvl.nbr > lvl.nol) return -1;
  loadLevel();
  return 0;
}


void initLevel()
{
  // Define Number of Levels
  int i = 0;
  char buf[128];
  do {
    sprintf(buf,"levels/%d.hlf",++i);
  } while (!loadFile(buf));
  lvl.nol = i-1;

  // Load the first one
  lvl.nbr = 1;
  loadLevel();
}

// EOF
