#include <stdlib.h>
#include <malloc.h>
#include <math.h>
#include <pspdisplay.h>
#include <psputils.h>
#include <png.h>
#include <pspgu.h>
#include <pspgum.h>

#include "graphics.h"
#include "framebuffer.h"

#define IS_ALPHA(color) (((color)&0xff000000)==0xff000000?0:1)
#define FRAMEBUFFER_SIZE (PSP_LINE_SIZE*SCR_H*4)
#define MAX(X, Y) ((X) > (Y) ? (X) : (Y))

extern u8 msx[];

unsigned int __attribute__((aligned(16))) list[262144];
static int dispBufferNumber;
static int initialized = 0;

int getNextPower2(int width)
{
  int b = width;
  int n;
  for (n = 0; b != 0; n++) b >>= 1;
  b = 1 << n;
  if (b == 2 * width) b >>= 1;
  return b;
}

Color* getVramDrawBuffer()
{
  Color* vram = (Color*) g_vram_base;
  if (dispBufferNumber == 0) vram += FRAMEBUFFER_SIZE / sizeof(Color);
  return vram;
}

Color* getVramDisplayBuffer()
{
  Color* vram = (Color*) g_vram_base;
  if (dispBufferNumber == 1) vram += FRAMEBUFFER_SIZE / sizeof(Color);
  return vram;
}

void user_warning_fn(png_structp png_ptr, png_const_charp warning_msg)
{
}

Image* loadImage(const char* filename, short swizzle)
{
  png_structp png_ptr;
  png_infop info_ptr;
  unsigned int sig_read = 0;
  png_uint_32 width, height;
  int bit_depth, color_type, interlace_type, x, y;
  u32* line;
  FILE *fp;
  Image* image = (Image*) malloc(sizeof(Image));
  if (!image) return NULL;

  if ((fp = fopen(filename, "rb")) == NULL) return NULL;
  png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
  if (png_ptr == NULL) {
    free(image);
    fclose(fp);
    return NULL;
}
  png_set_error_fn(png_ptr, (png_voidp) NULL, (png_error_ptr) NULL, user_warning_fn);
  info_ptr = png_create_info_struct(png_ptr);
  if (info_ptr == NULL) {
    free(image);
    fclose(fp);
    png_destroy_read_struct(&png_ptr, png_infopp_NULL, png_infopp_NULL);
    return NULL;
}
  png_init_io(png_ptr, fp);
  png_set_sig_bytes(png_ptr, sig_read);
  png_read_info(png_ptr, info_ptr);
  png_get_IHDR(png_ptr, info_ptr, &width, &height, &bit_depth, &color_type, &interlace_type, int_p_NULL, int_p_NULL);
  if (width > 512 || height > 512) {
    free(image);
    fclose(fp);
    png_destroy_read_struct(&png_ptr, png_infopp_NULL, png_infopp_NULL);
    return NULL;
}
  image->imgW = width;
  image->imgH = height;
  image->texW = getNextPower2(width);
  image->texH = getNextPower2(height);
  image->ratio = (float)width / height;
  png_set_strip_16(png_ptr);
  png_set_packing(png_ptr);
  if (color_type == PNG_COLOR_TYPE_PALETTE) png_set_palette_to_rgb(png_ptr);
  if (color_type == PNG_COLOR_TYPE_GRAY && bit_depth < 8) png_set_gray_1_2_4_to_8(png_ptr);
  if (png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS)) png_set_tRNS_to_alpha(png_ptr);
  png_set_filler(png_ptr, 0xff, PNG_FILLER_AFTER);
  image->data = (Color*) memalign(16, image->texW * image->texH * sizeof(Color));
  if (!image->data) {
    free(image);
    fclose(fp);
    png_destroy_read_struct(&png_ptr, png_infopp_NULL, png_infopp_NULL);
    return NULL;
}
  line = (u32*) malloc(width * 4);
  if (!line) {
    free(image->data);
    free(image);
    fclose(fp);
    png_destroy_read_struct(&png_ptr, png_infopp_NULL, png_infopp_NULL);
    return NULL;
}
  for (y = 0; y < height; y++) {
    png_read_row(png_ptr, (u8*) line, png_bytep_NULL);
    for (x = 0; x < width; x++) {
      u32 color = line[x];
      image->data[x + y * image->texW] =  color;
  }
}
  free(line);
  png_read_end(png_ptr, info_ptr);
  png_destroy_read_struct(&png_ptr, &info_ptr, png_infopp_NULL);
  fclose(fp);
  image->swizzled=0;
  if (swizzle) Swizzle(image);
  return image;
}

extern void blitScreenToImage(Image* dest)
{
  if (!initialized) return;
  Color* vram = getVramDisplayBuffer();
  sceGuCopyImage(GU_PSM_8888,0,0,dest->imgW,dest->imgH,getNextPower2(dest->imgW),vram,0,0,PSP_LINE_SIZE,dest->data);
}

extern void blitImagePartToScreen(int sx, int sy, int width, int height, Image* img, int dx, int dy)
{
  if (!initialized) return;
  if ((dx+width<0 || dy+height<0) || (dx>480 || dy>272)) return;

  sceGuTexMode(GU_PSM_8888,0,0,img->swizzled);
  sceGuTexImage(0, img->texW, img->texH, img->texW, (void*) img->data);
  sceGuEnable(GU_TEXTURE_2D);
  sceGuDisable(GU_DEPTH_TEST);
  sceGuEnable(GU_BLEND);
  int j = 0;
  while (j < width)
  {
    tVertex* v = sceGuGetMemory(2 * sizeof(tVertex));
    int sliceWidth = 64;
    if (j + sliceWidth > width) sliceWidth = width - j;
    v[0].u = sx + j;
    v[0].v = sy;
    v[0].x = dx + j;
    v[0].y = dy;
    v[0].z = 0;
    v[1].u = sx + j + sliceWidth;
    v[1].v = sy + height;
    v[1].x = dx + j + sliceWidth;
    v[1].y = dy + height;
    v[1].z = 0;
    sceGuDrawArray(GU_SPRITES, GU_TEXTURE_16BIT | GU_VERTEX_16BIT | GU_TRANSFORM_2D, 2, 0, v);
    j += sliceWidth;
  }
}

void blitAImageToScreen(Image* img, int dx, int dy, short alpha)
{
  if (!initialized || img == NULL) return;

  sceGuTexFilter(GU_LINEAR,GU_LINEAR);
  sceGuTexMode(GU_PSM_8888,0,0,img->swizzled);
  sceGuTexImage(0,img->texW,img->texH,
                img->texW,img->data);

  tVertex* v = sceGuGetMemory(2 * sizeof(tVertex));
  v[0].u = 0;
  v[0].v = 0;
  v[0].x = dx;
  v[0].y = dy;
  v[0].z = 0;
  v[1].u = img->imgW;
  v[1].v = img->imgH;
  v[1].x = dx+img->imgW;
  v[1].y = dy+img->imgH;
  v[1].z = 0;
  sceGuEnable(GU_TEXTURE_2D);
  sceGuDisable(GU_DEPTH_TEST);
  sceGuEnable(GU_BLEND);
  sceGuColor(GU_RGBA(255,255,255,alpha));
  sceGuDrawArray(GU_SPRITES, GU_TEXTURE_16BIT | GU_VERTEX_16BIT | GU_TRANSFORM_2D, 2, 0, v);
  sceGuColor(WHITE);
}

void blitCRImageToScreen(Image* img, int dx, int dy, short rot)
{
  if (!initialized || img == NULL) return;

  sceGuTexFilter(GU_NEAREST,GU_NEAREST);
  sceGuTexMode(GU_PSM_8888,0,0,img->swizzled);
  sceGuTexImage(0,img->texW,img->texH,
                img->texW,img->data);

  float sin = sinf(rot*GU_PI/180), cos = cosf(rot*GU_PI/180);
  tVertex* v = (tVertex*)sceGuGetMemory(4 * sizeof(tVertex));

  v[0].u = 0;
  v[0].v = 0;
  v[0].x = -img->imgW/2*cos + img->imgH/2*sin + dx;
  v[0].y = -img->imgH/2*cos - img->imgW/2*sin + dy;
  v[1].u = img->imgW-1;
  v[1].v = 0;
  v[1].x =  img->imgW/2*cos + img->imgH/2*sin + dx;
  v[1].y = -img->imgH/2*cos + img->imgW/2*sin + dy;
  v[2].u = img->imgW-1;
  v[2].v = img->imgH-1;
  v[2].x =  img->imgW/2*cos - img->imgH/2*sin + dx;
  v[2].y =  img->imgH/2*cos + img->imgW/2*sin + dy;
  v[3].u = 0;
  v[3].v = img->imgH-1;
  v[3].x = -img->imgW/2*cos - img->imgH/2*sin + dx;
  v[3].y =  img->imgH/2*cos - img->imgW/2*sin + dy;

  sceGuEnable(GU_TEXTURE_2D);
  sceGuDisable(GU_DEPTH_TEST);
  sceGuEnable(GU_BLEND);
  sceGuColor(0xFFFFFFFF);
  sceGumDrawArray(GU_TRIANGLE_FAN,GU_TEXTURE_16BIT|GU_VERTEX_16BIT|GU_TRANSFORM_2D,4,0,v); 
}

void blitImageToScreen(int dw, int dh, Image* img, int dx, int dy)
{
  if (!initialized || img == NULL) return;
  if ((dx+dw<0 || dy+dh<0) || (dx>480 || dy>272)) return;

  sceGuTexFilter(GU_LINEAR,GU_LINEAR);
  sceGuTexMode(GU_PSM_8888,0,0,img->swizzled);
  sceGuTexImage(0,img->texW,img->texH,
                img->texW,img->data);

  tVertex* v = sceGuGetMemory(2 * sizeof(tVertex));
  v[0].u = 0;
  v[0].v = 0;
  v[0].x = dx;
  v[0].y = dy;
  v[0].z = 0;
  v[1].u = img->imgW;
  v[1].v = img->imgH;
  v[1].x = dx+dw;
  v[1].y = dy+dh;
  v[1].z = 0;
  sceGuEnable(GU_TEXTURE_2D);
  sceGuDisable(GU_DEPTH_TEST);
  sceGuDrawArray(GU_SPRITES, GU_TEXTURE_16BIT | GU_VERTEX_16BIT | GU_TRANSFORM_2D, 2, 0, v);
}

Image* createImage(int width, int height)
{
  Image* image = (Image*) malloc(sizeof(Image));
  if (!image) return NULL;
  image->imgW = width;
  image->imgH = height;
  image->texW = getNextPower2(width);
  image->texH = getNextPower2(height);
  image->data = (Color*) memalign(16, image->texW * image->texH * sizeof(Color));
  if (!image->data) return NULL;
  memset(image->data, 0, image->texW * image->texH * sizeof(Color));
  return image;
}

void freeImage(Image* image)
{
  free(image->data);
  free(image);
}

void clearImage(Color color, Image* image)
{
  int i;
  int size = image->texW * image->texH;
  Color* data = image->data;
  for (i = 0; i < size; i++, data++) *data = color;
}

void clearScreen(Color color)
{
  if (!initialized) return;
  sceGuClearColor(color);
  sceGuClear(GU_COLOR_BUFFER_BIT|GU_DEPTH_BUFFER_BIT|GU_FAST_CLEAR_BIT);
}

Color getPixelScreen(int x, int y)
{
  Color* vram = getVramDrawBuffer();
  return vram[PSP_LINE_SIZE * y + x];
}

Color getPixelImage(int x, int y, Image* image)
{
  return image->data[x + y * image->texW];
}

void saveImage(const char* filename, Color* data, int width, int height, int lineSize, int saveAlpha)
{
  png_structp png_ptr;
  png_infop info_ptr;
  FILE* fp;
  int i, x, y;
  u8* line;
  
  if ((fp = fopen(filename, "wb")) == NULL) return;
  png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
  if (!png_ptr) return;
  info_ptr = png_create_info_struct(png_ptr);
  if (!info_ptr) {
    png_destroy_write_struct(&png_ptr, (png_infopp)NULL);
    return;
}
  png_init_io(png_ptr, fp);
  png_set_IHDR(png_ptr, info_ptr, width, height, 8,
    saveAlpha ? PNG_COLOR_TYPE_RGBA : PNG_COLOR_TYPE_RGB,
    PNG_INTERLACE_NONE, PNG_COMPRESSION_TYPE_DEFAULT, PNG_FILTER_TYPE_DEFAULT);
  png_write_info(png_ptr, info_ptr);
  line = (u8*) malloc(width * (saveAlpha ? 4 : 3));
  for (y = 0; y < height; y++) {
    for (i = 0, x = 0; x < width; x++) {
      Color color = data[x + y * lineSize];
      u8 r = color & 0xff; 
      u8 g = (color >> 8) & 0xff;
      u8 b = (color >> 16) & 0xff;
      u8 a = saveAlpha ? (color >> 24) & 0xff : 0xff;
      line[i++] = r;
      line[i++] = g;
      line[i++] = b;
      if (saveAlpha) line[i++] = a;
  }
    png_write_row(png_ptr, line);
}
  free(line);
  png_write_end(png_ptr, info_ptr);
  png_destroy_write_struct(&png_ptr, (png_infopp)NULL);
  fclose(fp);
}

void flipScreen()
{
  if (!initialized) return;
  sceGuSwapBuffers();
  dispBufferNumber ^= 1;
}

#define BUF_WIDTH (512)
#define SCR_WIDTH (480)
#define SCR_HEIGHT (272)
#define PIXEL_SIZE (4) /* change this if you change to another screenmode */
#define FRAME_SIZE (BUF_WIDTH * SCR_HEIGHT * PIXEL_SIZE)
#define ZBUF_SIZE (BUF_WIDTH SCR_HEIGHT * 2) /* zbuffer seems to be 16-bit? */

void initGraphics()
{
  dispBufferNumber = 0;

  sceGuInit();

  guStart();
  sceGuDrawBuffer(GU_PSM_8888, (void*)FRAMEBUFFER_SIZE, PSP_LINE_SIZE);
  sceGuDispBuffer(SCR_W, SCR_H, (void*)0, PSP_LINE_SIZE);
  sceGuClear(GU_COLOR_BUFFER_BIT | GU_DEPTH_BUFFER_BIT);
  sceGuDepthBuffer((void*) (FRAMEBUFFER_SIZE*2), PSP_LINE_SIZE);
  sceGuOffset(2048 - (SCR_W / 2), 2048 - (SCR_H / 2));
  sceGuViewport(2048, 2048, SCR_W, SCR_H);
  sceGuDepthRange(0xc350, 0x2710);
  sceGuScissor(0, 0, SCR_W, SCR_H);
  sceGuEnable(GU_SCISSOR_TEST);
  sceGuAlphaFunc(GU_GREATER, 0, 0xff);
  sceGuEnable(GU_ALPHA_TEST);
  sceGuDepthFunc(GU_GEQUAL);
  sceGuDisable(GU_DEPTH_TEST);
  sceGuFrontFace(GU_CW);
  sceGuShadeModel(GU_SMOOTH);
  sceGuEnable(GU_CULL_FACE);
  sceGuEnable(GU_TEXTURE_2D);
  sceGuEnable(GU_CLIP_PLANES);
  sceGuTexMode(GU_PSM_8888, 0, 0, 0);
  sceGuTexFunc(GU_TFX_MODULATE, GU_TCC_RGBA);
  sceGuTexFilter(GU_NEAREST, GU_NEAREST);
  sceGuAmbientColor(0xffffffff);
  sceGuEnable(GU_BLEND);
  sceGuBlendFunc(GU_ADD, GU_SRC_ALPHA, GU_ONE_MINUS_SRC_ALPHA, 0, 0);
  sceGuFinish();
  sceGuSync(0, 0);

  sceDisplayWaitVblankStart();
  sceGuDisplay(GU_TRUE);

  initialized = 1;
}

void disableGraphics()
{
  sceGuTerm();
  initialized = 0;
}

void guStart()
{
  sceGuStart(GU_DIRECT, list);
}

extern void drawFilledRect(Color color, int x, int y, int w, int h)
{
  Vertex *v = sceGuGetMemory(2 * sizeof(Vertex));
  v[0].x = x;
  v[0].y = y;
  v[1].x = x + w;
  v[1].y = y + h;
  sceGuDisable(GU_TEXTURE_2D);
  sceGuEnable(GU_BLEND);
  sceGuDisable(GU_DEPTH_TEST);
  sceGuColor(color);
  sceGuDrawArray(GU_SPRITES,GU_VERTEX_16BIT|GU_TRANSFORM_2D,2,0,v);
  sceGuColor(WHITE);
}

extern void drawRect(Color color, int x, int y, int w, int h)
{
  Vertex *v = sceGuGetMemory(8 * sizeof(Vertex));
  v[7].x = x;
  v[7].y = y;
  v[0] = v[7];
  v[1].x = x + w - 1;
  v[1].y = y ;
  v[2] = v[1];
  v[3].x = x + w - 1;
  v[3].y = y + h - 1;
  v[4] = v[3];
  v[5].x = x;
  v[5].y = y + h - 1;
  v[6] = v[5];
  sceGuDisable(GU_TEXTURE_2D);
  sceGuEnable(GU_BLEND);
  sceGuDisable(GU_DEPTH_TEST);
  sceGuColor(color);
  sceGuDrawArray(GU_LINES,GU_VERTEX_16BIT|GU_TRANSFORM_2D,8,0,v);
  sceGuColor(WHITE);
}

void drawLineScreen(Color color, int sx, int sy, int dx, int dy)
{
  Vertex *v = sceGuGetMemory(2 * sizeof(Vertex));
  v[0].x = sx;
  v[0].y = sy;
  v[0].z = 0;
  v[1].x = dx;
  v[1].y = dy;
  v[1].z = 0;
  sceGuEnable(GU_BLEND);
  sceGuDisable(GU_TEXTURE_2D);
  sceGuDisable(GU_DEPTH_TEST);
  sceGuColor(color);
  sceGuDrawArray(GU_LINES,GU_VERTEX_16BIT|GU_TRANSFORM_2D,2,0,v);
  sceGuColor(WHITE);
}

void putPixelScreen(Color color, int x, int y)
{
  cVertex *v = sceGuGetMemory(sizeof(cVertex));
  v[0].x = x;
  v[0].y = y;
  v[0].z = 0;
  v[0].color = color;
  sceGuEnable(GU_BLEND);
  sceGuDisable(GU_TEXTURE_2D);
  sceGuDisable(GU_DEPTH_TEST);
  sceGuDrawArray(GU_POINTS,GU_COLOR_8888|GU_VERTEX_16BIT|GU_TRANSFORM_2D,1,0,v);
}

void drawVShade(Color c1, Color c2, int x, int y, int w, int h)
{
  cVertex *v = sceGuGetMemory(4 * sizeof(cVertex));
  v[0].x = x;
  v[0].y = y;
  v[0].color = c1;
  v[1].x = x + w;
  v[1].y = y;
  v[1].color = c1;
  v[2].x = x + w;
  v[2].y = y + h;
  v[2].color = c2;
  v[3].x = x;
  v[3].y = y + h;
  v[3].color = c2;
  sceGuEnable(GU_BLEND);
  sceGuDisable(GU_TEXTURE_2D);
  sceGuDisable(GU_DEPTH_TEST);
  sceGuDrawArray(GU_TRIANGLE_FAN, GU_COLOR_8888 | GU_VERTEX_16BIT |
                 GU_TRANSFORM_2D, 4, 0, v);
}

void blur(int x, int y, int w, int h, float factor)
{
  if (!initialized) return;

  sceGuTexFilter(GU_LINEAR,GU_LINEAR);
  tVertex* v = sceGuGetMemory(3 * 2 * sizeof(tVertex));
  sceGuTexMode(GU_PSM_8888,0,0,0);
  sceGuTexImage(0,512,512,512,getVramDrawBuffer());
  v[0].u = x,                v[0].v = y;
  v[0].x = x,                v[0].y = y;
  v[1].u = x + w,            v[1].v = y + h;
  v[1].x = x + w/factor,     v[1].y = y + h/factor;

  v[2].u = x,                v[2].v = y;
  v[2].x = x + w - w/factor, v[2].y = y + h - h/factor;
  v[3].u = x + w/factor,     v[3].v = y + h/factor;
  v[3].x = x + w,            v[3].y = y + h;

  v[4].u = x + w - w/factor, v[4].v = y + h - h/factor;
  v[4].x = x,                v[4].y = y;
  v[5].u = x + w,            v[5].v = y + h;
  v[5].x = x + w,            v[5].y = y + h;
  sceGuDisable(GU_BLEND);
  sceGuEnable(GU_TEXTURE_2D);
  sceGuDisable(GU_DEPTH_TEST);
  sceGuDrawArray(GU_SPRITES, GU_TEXTURE_16BIT | GU_VERTEX_16BIT | GU_TRANSFORM_2D, 3*2, 0, v);
  sceGuEnable(GU_BLEND);
}

extern void swizzle_fast(unsigned char *dest, unsigned char *source, int width, int height)
{
  int i,j;
  int rowblocks = (width / 16);
  int rowblocks_add = (rowblocks-1)*128;
  unsigned int block_address = 0;
  unsigned int *img = (unsigned int*)source;
  for (j = 0; j < height; j++)
  {
    unsigned int *block = (unsigned int*)((unsigned int)&dest[block_address] | 0x40000000);
    for (i = 0; i < rowblocks; i++)
    {
      *block++ = *img++;
      *block++ = *img++;
      *block++ = *img++;
      *block++ = *img++;
      block += 28;
    }
    if ((j&0x7)==0x7)
    {
      block_address += rowblocks_add;
    }
    block_address+=16;
  }
}
 
extern void Swizzle(Image* img)
{
  sceKernelDcacheWritebackAll();
  if(img->swizzled || img->imgH+img->imgW<32) return;
  else
  {
    int type = 4; // GU_PSM_8888
    long size = img->texW * img->texH * type;
    u8* temp = (u8*)malloc(size);
    swizzle_fast( temp, (u8*)img->data, (img->texW * type), img->texH );
    free(img->data);
    img->data = (u32*)temp;
    img->swizzled = GU_TRUE;
  }
  sceKernelDcacheWritebackAll();
}
