#ifndef GRAPHICS_H
#define GRAPHICS_H

#include <psptypes.h>

#define PSP_LINE_SIZE 512
#define SCR_W 480
#define SCR_H 272

typedef u32 Color;
#define RGB(r,g,b) ((r)|((g)<<8)|((b)<<16))
#define RGBA(r,g,b,a) ((r)|((g)<<8)|((b)<<16)|((a)<<24))
#define A(color) ((u8)(color >> 24 & 0xFF))
#define B(color) ((u8)(color >> 16 & 0xFF))
#define G(color) ((u8)(color >> 8 & 0xFF))
#define R(color) ((u8)(color & 0xFF))
enum colors
{
  LITERED  = 0xFF5858FF,
  TRED     = 0x8F0000FF,
  RED      = 0xFF0000FF,
  GREEN    = 0xFF00FF00,
  BLUE     = 0xFFFF0000,
  LITEBLUE = 0xB4FF9628,
  WHITE    = 0xFFFFFFFF,
  LITEGRAY = 0xFFBFBFBF,
  GRAY     = 0xFF7F7F7F,
  DARKGRAY = 0xFF3F3F3F,  
  BLACK    = 0xFF000000,
  TBLACK   = 0x8F000000,
  YELLOW   = 0xFF00FFFF
};


typedef struct
{
  unsigned short u, v;
  short x, y, z;
} tVertex;

typedef struct
{
  unsigned int color;
  short x,y,z;
} cVertex;

typedef struct
{
  short x,y,z;
} Vertex;

typedef struct
{
  float x,y,z;
} fVertex;

typedef struct
{
  unsigned short u, v;
  unsigned int color;
  short x, y, z;
} tcVertex;

typedef struct
{
  unsigned short u, v;
  unsigned int color;
  float x, y, z;
} tcfVertex;

typedef struct
{
  unsigned short u, v;
  float x, y, z;
} tfVertex;

typedef struct
{
  int texW;  // the real width of data, 2^n with n>=0
  int texH;  // the real height of data, 2^n with n>=0
  int imgW;
  int imgH;
  float ratio;
  char swizzled; // Boolean
  Color* data;
} Image;

int getNextPower2(int width);

void initGraphics();
void disableGraphics();
void guStart();
void clearScreen(Color color);
void flipScreen();

void blitImagePartToScreen(int sx, int sy, int width, int height, Image* source, int dx, int dy);
void blitAImageToScreen(Image* source, int dx, int dy, short alpha);
void blitImageToScreen(int dw, int dh, Image* source, int dx, int dy);
void blitCRImageToScreen(Image* src, int dx, int dy, short rot);
Color getPixelScreen(int x, int y);
void drawFilledRect(Color color, int x, int y, int width, int height);
void drawRect(Color color, int x, int y, int width, int height);
void drawLineScreen(Color color, int sx, int sy, int dx, int dy);
void putPixelScreen(Color color, int x, int y);
void drawVShade(Color c1, Color c2, int x, int y, int w, int h);
void blur(int x, int y, int w, int h, float factor);

Image* loadImage(const char* filename, short swizzle);
Image* createImage(int width, int height);
void freeImage(Image* image);
void clearImage(Color color, Image* image);
Color getPixelImage(int x, int y, Image* image);
void saveImage(const char* filename, Color* data, int width, int height, int lineSize, int saveAlpha);
void blitScreenToImage(Image* dest);

Color* getVramDrawBuffer();
Color* getVramDisplayBuffer();
void Swizzle(Image* img);

#endif
